@@ -105,11 +105,11 @@ fn build_v8() {
   }
 
   if let Some(p) = env::var_os("SCCACHE") {
-    cc_wrapper(&mut gn_args, &Path::new(&p));
+    cc_wrapper(&mut gn_args, Path::new(&p));
   } else if let Ok(p) = which("sccache") {
     cc_wrapper(&mut gn_args, &p);
   } else if let Some(p) = env::var_os("CCACHE") {
-    cc_wrapper(&mut gn_args, &Path::new(&p));
+    cc_wrapper(&mut gn_args, Path::new(&p));
   } else if let Ok(p) = which("ccache") {
     cc_wrapper(&mut gn_args, &p);
   } else {
