@@ -312,6 +312,10 @@ void v8__Isolate__SetWasmStreamingCallback(v8::Isolate* isolate,
   isolate->SetWasmStreamingCallback(callback);
 }
 
+bool v8__Isolate__HasPendingBackgroundTasks(v8::Isolate* isolate) {
+  return isolate->HasPendingBackgroundTasks();
+}
+
 void v8__Isolate__CreateParams__CONSTRUCT(
     uninit_t<v8::Isolate::CreateParams>* buf) {
   construct_in_place<v8::Isolate::CreateParams>(buf);
@@ -2118,6 +2122,11 @@ void v8_inspector__V8Inspector__contextCreated(
       ptr_to_local(&context), contextGroupId, humanReadableName));
 }
 
+bool v8_inspector__V8InspectorSession__canDispatchMethod(
+    v8_inspector::StringView method) {
+  return v8_inspector::V8InspectorSession::canDispatchMethod(method);
+}
+
 void v8_inspector__V8InspectorSession__DELETE(
     v8_inspector::V8InspectorSession* self) {
   delete self;
