@@ -6,14 +6,19 @@
 
 #include <cstdarg>
 
+#include "include/v8-metrics.h"
 #include "src/base/atomic-utils.h"
+#include "src/base/strings.h"
 #include "src/common/globals.h"
 #include "src/execution/isolate.h"
 #include "src/execution/thread-id.h"
+#include "src/heap/cppgc-js/cpp-heap.h"
+#include "src/heap/cppgc/metric-recorder.h"
 #include "src/heap/heap-inl.h"
 #include "src/heap/incremental-marking.h"
 #include "src/heap/spaces.h"
 #include "src/logging/counters.h"
+#include "src/logging/metrics.h"
 #include "src/logging/tracing-flags.h"
 #include "src/tracing/tracing-category-observer.h"
 
@@ -118,7 +123,6 @@ const char* GCTracer::Scope::Name(ScopeId id) {
   }
 #undef CASE
   UNREACHABLE();
-  return nullptr;
 }
 
 bool GCTracer::Scope::NeedsYoungEpoch(ScopeId id) {
@@ -445,6 +449,8 @@ void GCTracer::NotifySweepingCompleted() {
     heap_->code_space()->PrintAllocationsOrigins();
     heap_->map_space()->PrintAllocationsOrigins();
   }
+  metrics_report_pending_ = true;
+  NotifyGCCompleted();
 }
 
 void GCTracer::SampleAllocation(double current_ms,
@@ -513,6 +519,7 @@ void GCTracer::AddIncrementalMarkingStep(double duration, size_t bytes) {
     incremental_marking_bytes_ += bytes;
     incremental_marking_duration_ += duration;
   }
+  ReportIncrementalMarkingStepToRecorder();
 }
 
 void GCTracer::Output(const char* format, ...) const {
@@ -525,10 +532,10 @@ void GCTracer::Output(const char* format, ...) const {
 
   const int kBufferSize = 256;
   char raw_buffer[kBufferSize];
-  Vector<char> buffer(raw_buffer, kBufferSize);
+  base::Vector<char> buffer(raw_buffer, kBufferSize);
   va_list arguments2;
   va_start(arguments2, format);
-  VSNPrintF(buffer, format, arguments2);
+  base::VSNPrintF(buffer, format, arguments2);
   va_end(arguments2);
 
   heap_->AddToRingBuffer(buffer.begin());
@@ -1148,7 +1155,7 @@ bool GCTracer::SurvivalEventsRecorded() const {
 void GCTracer::ResetSurvivalEvents() { recorded_survival_ratios_.Reset(); }
 
 void GCTracer::NotifyIncrementalMarkingStart() {
-  incremental_marking_start_time_ = heap_->MonotonicallyIncreasingTimeInMs();
+  incremental_marking_start_time_ = MonotonicallyIncreasingTimeInMs();
 }
 
 void GCTracer::FetchBackgroundMarkCompactCounters() {
@@ -1295,5 +1302,147 @@ void GCTracer::RecordGCSumCounters(double atomic_pause_duration) {
                        "background_duration", marking_background_duration);
 }
 
+void GCTracer::NotifyGCCompleted() {
+  // Report full GC cycle metric to recorder only when both v8 and cppgc (if
+  // available) GCs have finished. This method is invoked by both v8 and cppgc.
+  if (!metrics_report_pending_) {
+    // V8 sweeping is not done yet.
+    return;
+  }
+  const auto* cpp_heap = heap_->cpp_heap();
+  if (cpp_heap &&
+      !CppHeap::From(cpp_heap)->GetMetricRecorder()->MetricsReportPending()) {
+    // Cppgc sweeping is not done yet.
+    return;
+  }
+  ReportFullCycleToRecorder();
+}
+
+namespace {
+
+void CopyTimeMetrics(
+    ::v8::metrics::GarbageCollectionPhases& metrics,
+    const cppgc::internal::MetricRecorder::FullCycle::IncrementalPhases&
+        cppgc_metrics) {
+  DCHECK_NE(-1, cppgc_metrics.mark_duration_us);
+  metrics.mark_wall_clock_duration_in_us = cppgc_metrics.mark_duration_us;
+  DCHECK_NE(-1, cppgc_metrics.sweep_duration_us);
+  metrics.sweep_wall_clock_duration_in_us = cppgc_metrics.sweep_duration_us;
+}
+
+void CopyTimeMetrics(
+    ::v8::metrics::GarbageCollectionPhases& metrics,
+    const cppgc::internal::MetricRecorder::FullCycle::Phases& cppgc_metrics) {
+  DCHECK_NE(-1, cppgc_metrics.compact_duration_us);
+  metrics.compact_wall_clock_duration_in_us = cppgc_metrics.compact_duration_us;
+  DCHECK_NE(-1, cppgc_metrics.mark_duration_us);
+  metrics.mark_wall_clock_duration_in_us = cppgc_metrics.mark_duration_us;
+  DCHECK_NE(-1, cppgc_metrics.sweep_duration_us);
+  metrics.sweep_wall_clock_duration_in_us = cppgc_metrics.sweep_duration_us;
+  DCHECK_NE(-1, cppgc_metrics.weak_duration_us);
+  metrics.weak_wall_clock_duration_in_us = cppgc_metrics.weak_duration_us;
+}
+
+void CopySizeMetrics(
+    ::v8::metrics::GarbageCollectionSizes& metrics,
+    const cppgc::internal::MetricRecorder::FullCycle::Sizes& cppgc_metrics) {
+  DCHECK_NE(-1, cppgc_metrics.after_bytes);
+  metrics.bytes_after = cppgc_metrics.after_bytes;
+  DCHECK_NE(-1, cppgc_metrics.before_bytes);
+  metrics.bytes_before = cppgc_metrics.before_bytes;
+  DCHECK_NE(-1, cppgc_metrics.freed_bytes);
+  metrics.bytes_freed = cppgc_metrics.freed_bytes;
+}
+
+::v8::metrics::Recorder::ContextId GetContextId(
+    v8::internal::Isolate* isolate) {
+  DCHECK_NOT_NULL(isolate);
+  if (isolate->context().is_null())
+    return v8::metrics::Recorder::ContextId::Empty();
+  HandleScope scope(isolate);
+  return isolate->GetOrRegisterRecorderContextId(isolate->native_context());
+}
+
+void FlushBatchedIncrementalEvents(
+    v8::metrics::GarbageCollectionFullMainThreadBatchedIncrementalMark&
+        batched_events,
+    Isolate* isolate) {
+  DCHECK_NOT_NULL(isolate->metrics_recorder());
+  DCHECK(!batched_events.events.empty());
+  isolate->metrics_recorder()->AddMainThreadEvent(std::move(batched_events),
+                                                  GetContextId(isolate));
+}
+
+}  // namespace
+
+void GCTracer::ReportFullCycleToRecorder() {
+  const std::shared_ptr<metrics::Recorder>& recorder =
+      heap_->isolate()->metrics_recorder();
+  DCHECK_NOT_NULL(recorder);
+  if (!recorder->HasEmbedderRecorder()) return;
+  if (!incremental_mark_batched_events_.events.empty()) {
+    FlushBatchedIncrementalEvents(incremental_mark_batched_events_,
+                                  heap_->isolate());
+  }
+  v8::metrics::GarbageCollectionFullCycle event;
+  if (heap_->cpp_heap()) {
+    auto* cpp_heap = v8::internal::CppHeap::From(heap_->cpp_heap());
+    cpp_heap->GetMetricRecorder()->FlushBatchedIncrementalEvents();
+    const base::Optional<cppgc::internal::MetricRecorder::FullCycle>
+        optional_cppgc_event =
+            cpp_heap->GetMetricRecorder()->ExtractLastFullGcEvent();
+    DCHECK(optional_cppgc_event.has_value());
+    const cppgc::internal::MetricRecorder::FullCycle& cppgc_event =
+        optional_cppgc_event.value();
+    CopyTimeMetrics(event.total_cpp, cppgc_event.total);
+    CopyTimeMetrics(event.main_thread_cpp, cppgc_event.main_thread);
+    CopyTimeMetrics(event.main_thread_atomic_cpp,
+                    cppgc_event.main_thread_atomic);
+    CopyTimeMetrics(event.main_thread_incremental_cpp,
+                    cppgc_event.main_thread_incremental);
+    CopySizeMetrics(event.objects_cpp, cppgc_event.objects);
+    CopySizeMetrics(event.memory_cpp, cppgc_event.memory);
+    DCHECK_NE(-1, cppgc_event.collection_rate_in_percent);
+    event.collection_rate_cpp_in_percent =
+        cppgc_event.collection_rate_in_percent;
+    DCHECK_NE(-1, cppgc_event.efficiency_in_bytes_per_us);
+    event.efficiency_cpp_in_bytes_per_us =
+        cppgc_event.efficiency_in_bytes_per_us;
+    DCHECK_NE(-1, cppgc_event.main_thread_efficiency_in_bytes_per_us);
+    event.main_thread_efficiency_cpp_in_bytes_per_us =
+        cppgc_event.main_thread_efficiency_in_bytes_per_us;
+  }
+  // TODO(chromium:1154636): Populate v8 metrics.
+  recorder->AddMainThreadEvent(event, GetContextId(heap_->isolate()));
+  metrics_report_pending_ = false;
+}
+
+void GCTracer::ReportIncrementalMarkingStepToRecorder() {
+  static constexpr int kMaxBatchedEvents =
+      CppHeap::MetricRecorderAdapter::kMaxBatchedEvents;
+  const std::shared_ptr<metrics::Recorder>& recorder =
+      heap_->isolate()->metrics_recorder();
+  DCHECK_NOT_NULL(recorder);
+  if (!recorder->HasEmbedderRecorder()) return;
+  incremental_mark_batched_events_.events.emplace_back();
+  if (heap_->cpp_heap()) {
+    const base::Optional<
+        cppgc::internal::MetricRecorder::MainThreadIncrementalMark>
+        cppgc_event = v8::internal::CppHeap::From(heap_->cpp_heap())
+                          ->GetMetricRecorder()
+                          ->ExtractLastIncrementalMarkEvent();
+    if (cppgc_event.has_value()) {
+      DCHECK_NE(-1, cppgc_event.value().duration_us);
+      incremental_mark_batched_events_.events.back()
+          .cpp_wall_clock_duration_in_us = cppgc_event.value().duration_us;
+    }
+  }
+  // TODO(chromium:1154636): Populate event.wall_clock_duration_in_us.
+  if (incremental_mark_batched_events_.events.size() == kMaxBatchedEvents) {
+    FlushBatchedIncrementalEvents(incremental_mark_batched_events_,
+                                  heap_->isolate());
+  }
+}
+
 }  // namespace internal
 }  // namespace v8
